using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using realm = RoboRealm.Proxy;
using W3C.Soap;

namespace RoboRealmVPL
{
	[Contract(Contract.Identifier)]
	[DisplayName("RoboRealmVPL")]
	[Description("RoboRealmVPL service (no description provided)")]
	class RoboRealmVPLService : DsspServiceBase
	{
        [Partner("realm", Contract = realm.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExistingOrCreate)]
        private realm.InterfaceOperations _realmPort = new realm.InterfaceOperations();
        private realm.InterfaceOperations _realmNotify = new realm.InterfaceOperations();
        
        [ServiceState]
		RoboRealmVPLState _state = new RoboRealmVPLState();
		
		[ServicePort("/RoboRealmVPL", AllowMultipleInstances = true)]
		RoboRealmVPLOperations _mainPort = new RoboRealmVPLOperations();
		
		public RoboRealmVPLService(DsspServiceCreationPort creationPort)
			: base(creationPort)
		{
		}
		
		protected override void Start()
		{
			base.Start();

            // Add service specific initialization here.
            _realmPort.Subscribe(_realmNotify);

            realm.SetCaptureRequest req = new realm.SetCaptureRequest();
            req.captureImage = false;
            req.captureVariable = true;
            req.Hostname = "localhost";
            req.Port = 6060;
            _realmPort.SetCapture(req);
        }

        public IEnumerator<ITask> Execute(String txt)
        {
            if (txt == null) txt = "<Center_of_Gravity></Center_of_Gravity>";
            // load in a simple program into roborealm to generate some variables
            realm.ExecuteProgramRequest program = new realm.ExecuteProgramRequest();
            // note that this config does not include any display elements in
            // roborealm so you will not see the COG in RoboRealm. You can remove the next
            // lines if you just want to use the existing program in RoboRealm.
            program.Source = txt;

            _realmPort.ExecuteProgram(program);

            return null;
        }

        public IEnumerator<ITask> GetVariable(String txt)
        {
            // called when a new frame has been captured .. this means the variables
            // may have changed. Create a query variable request to request specific
            // variables.
            realm.QueryVariablesRequest req = new realm.QueryVariablesRequest();
            req._names = new List<String>();
            req._names.Add(txt);
            if (req != null)
            {
                // send of the request and process the results
                yield return Arbiter.Choice(
                    _realmPort.QueryVariables(req),
                    delegate(realm.QueryVariablesResponse res)
                    {
                        // for now just print the results in the command console. You could
                        // use these values to futher partner with a robotic drive system
                        // and move the robot based on COG values.
                        Console.WriteLine("COG: " + res._values[0] + "," + res._values[1]);
                    },
                    delegate(Fault f)
                    {
                    });
            }
        }
	}
}


